# 12 "java/java_lexer.mll"
 

  open Java_parser
  open Lexing
  open Java_ast

  type location = position * position

  let loc lexbuf : location = 
    (lexeme_start_p lexbuf, lexeme_end_p lexbuf)

  exception Lexical_error of location * string
  exception Syntax_error of location
  exception Dotdot of string


  let lex_error lexbuf s = raise (Lexical_error(loc lexbuf,s))

  let buf = Buffer.create 97

  let kw_table = 
    let table = Hashtbl.create 17 in
    let _ = 
      List.iter
	(fun (s,t) -> Hashtbl.add table s t)
	[ "abstract", ABSTRACT;
	  "assigns", ASSIGNS;
	  "axiom", AXIOM;
	  "behavior", BEHAVIOR;
	  "boolean", BOOLEAN;
	  "break", BREAK;
	  "byte", BYTE;
	  "byvalue", BYVALUE;
	  "case", CASE;
	  "cast", CAST;
	  "catch", CATCH;
	  "char", CHAR;
	  "class", CLASS;
	  "const", CONST;
	  "continue", CONTINUE;
	  "decreases", DECREASES;
	  "default", DEFAULT;
	  "do", DO;
	  "double", DOUBLE;
	  "else", ELSE;
	  "ensures", ENSURES;
	  "extends", EXTENDS;
	  "false", FALSE;
	  "final", FINAL;
	  "finally", FINALLY;
	  "float", FLOAT;
	  "for", FOR;
	  "future", FUTURE;
	  "generic", GENERIC;
	  "goto", GOTO;
	  "if", IF;
	  "implements", IMPLEMENTS;
	  "import", IMPORT;
	  "inner", INNER;
	  "instanceof", INSTANCEOF;
	  "int", INT;
	  "interface", INTERFACE;
	  "invariant", INVARIANT;
	  "long", LONG;
	  "loop_invariant", LOOP_INVARIANT;
	  "native", NATIVE;
	  "new", NEW;
	  "null", NULL;
	  "operator", OPERATOR;
	  "outer", OUTER;
	  "package", PACKAGE;
	  "private", PRIVATE;
	  "protected", PROTECTED;
	  "public", PUBLIC;
	  "requires", REQUIRES;
	  "rest", REST;
	  "return", RETURN;
	  "short", SHORT;
	  "signals", SIGNALS;
	  "static", STATIC;
	  "super", SUPER;
	  "switch", SWITCH;
	  "synchronized", SYNCHRONIZED;
	  "this", THIS;
	  (* "threadsafe" ? *)
	  "throw", THROW;
	  "throws", THROWS;
	  "transient", TRANSIENT;
	  "true", TRUE;
	  "try", TRY;
	  "var", VAR;
	  "void", VOID;
	  "volatile", VOLATILE;
	  "while", WHILE;	
	]
    in table

  let id_or_kw s =
    try
      let k = Hashtbl.find kw_table s in
      (*i
	prerr_string "Keyword ";
	prerr_endline s;
	i*)
      k
    with
	Not_found ->
	  (*i
	    prerr_string "Ident ";
	    prerr_endline s;
	    i*)
	  (ID s)

  let special_kw_table = 
    let table = Hashtbl.create 17 in
    let _ = 
      List.iter
	(fun (s,t) -> Hashtbl.add table s t)
	[ (* "everything", BSEVERYTHING ; *)
	  "exists", BSEXISTS ;
	  (* "fresh", BSFRESH ; *)
	  "forall", BSFORALL ;
	  "nothing", BSNOTHING;
	  (*
	  "fields_of", BSFIELDSOF;
          "not_conditionally_updated", BSNOTCONDITIONALLYUPDATED;
	  *)
	  "old", BSOLD;
	  "result", BSRESULT;
	  (*"type", BSTYPE;
	  "typeof", BSTYPEOF;
	  "fpi", BSFPI; *)
	]
    in table

  let special_id lexbuf s =
    try
      Hashtbl.find special_kw_table s
    with
	Not_found ->
	  lex_error lexbuf ("unknown special keyword \\"^s)

(*

  let jml_spec_token base jml_string =
(*i
    Format.fprintf Config.log "In file %s, parsing JML Spec: %s@."
      (Location.base_filename base) jml_string;
i*)
    match Jml_syntax.parse_jml_specification base jml_string with 
    | Ast_types.Jml_declaration d -> JML_DECLARATIONS(d)
    | Ast_types.Jml_method_specification s -> JML_METHOD_SPECIFICATION(s)
    | Ast_types.Jml_loop_annotation la -> JML_LOOP_ANNOTATION(la)
    | Ast_types.Jml_assertion a -> JML_ASSERTION(a)
    | Ast_types.Jml_annotation_statement Ast_types.Set_statement s -> JML_ANNOTATION_STATEMENT(Ast_types.Set_statement(s))
    | Ast_types.Jml_axiom(id,e) -> JML_AXIOM(id,e)
    | Ast_types.Jml_type(id) -> JML_TYPE(id)
    | Ast_types.Jml_logic_reads(id,t,p,r) -> JML_LOGIC_READS(id,t,p,r)
    | Ast_types.Jml_logic_def(id,t,p,e) -> JML_LOGIC_DEF(id,t,p,e)
    assert false

*)
  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- 
      { pos with pos_lnum = pos.pos_lnum + 1; 
	  pos_bol = pos.pos_cnum }


# 172 "java/java_lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\189\255\190\255\191\255\192\255\193\255\194\255\195\255\
    \196\255\197\255\004\000\081\000\110\000\078\000\081\000\228\255\
    \229\255\003\000\231\255\067\000\082\000\084\000\108\000\086\000\
    \087\000\107\000\110\000\171\000\246\255\247\255\196\000\254\000\
    \130\000\254\255\003\000\222\255\139\000\087\000\253\255\250\255\
    \001\000\144\000\252\255\080\001\155\001\202\255\186\000\165\001\
    \188\001\220\255\242\255\219\255\241\255\223\255\221\255\215\255\
    \235\255\213\255\234\255\208\255\233\255\207\255\214\255\210\255\
    \094\000\218\255\209\255\112\000\217\255\109\000\216\255\221\001\
    \231\001\000\002\030\002\057\002\206\255\205\255\086\002\204\255\
    \203\255\118\002\147\002\241\001\157\002\010\002\201\255\136\000\
    \182\002\185\002\137\000\139\000\199\255\252\002\019\003\042\003\
    \141\000\198\255\200\255\058\002\021\003\022\003\043\003\255\255\
    \171\000\251\255\173\000\137\000\181\002\182\002\061\002\138\000\
    \249\002";
  Lexing.lex_backtrk = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\065\000\049\000\049\000\043\000\044\000\255\255\
    \255\255\025\000\255\255\023\000\031\000\019\000\018\000\017\000\
    \015\000\012\000\011\000\010\000\255\255\255\255\007\000\065\000\
    \016\000\255\255\000\000\255\255\255\255\004\000\255\255\255\255\
    \255\255\255\255\255\255\006\000\053\000\255\255\255\255\053\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \028\000\255\255\255\255\029\000\255\255\030\000\255\255\255\255\
    \052\000\255\255\255\255\050\000\255\255\255\255\051\000\255\255\
    \255\255\255\255\052\000\255\255\054\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\055\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\004\000\003\000\255\255\255\255\
    \255\255\255\255\002\000\003\000\255\255\003\000\001\000\004\000\
    \255\255";
  Lexing.lex_default = 
   "\002\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\087\000\255\255\255\255\255\255\255\255\000\000\
    \000\000\255\255\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\255\255\255\255\
    \255\255\000\000\255\255\000\000\040\000\255\255\000\000\000\000\
    \040\000\041\000\000\000\255\255\255\255\000\000\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255\000\000\000\000\255\255\000\000\255\255\000\000\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\255\255\000\000\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\255\255\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\000\000\000\000\100\000\100\000\100\000\038\000\000\000\
    \106\000\000\000\106\000\255\255\109\000\109\000\255\255\255\255\
    \255\255";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\034\000\033\000\039\000\034\000\034\000\000\000\000\000\
    \034\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \034\000\019\000\003\000\034\000\000\000\023\000\022\000\010\000\
    \009\000\008\000\024\000\026\000\028\000\025\000\027\000\032\000\
    \012\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\015\000\029\000\014\000\020\000\013\000\016\000\
    \062\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\005\000\031\000\004\000\017\000\030\000\
    \088\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\007\000\021\000\006\000\018\000\072\000\
    \061\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\066\000\067\000\064\000\063\000\059\000\
    \060\000\057\000\056\000\054\000\053\000\039\000\071\000\038\000\
    \052\000\050\000\042\000\065\000\072\000\076\000\075\000\075\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\073\000\073\000\
    \051\000\055\000\070\000\049\000\037\000\068\000\069\000\098\000\
    \092\000\036\000\092\000\071\000\097\000\033\000\071\000\255\255\
    \103\000\103\000\076\000\000\000\000\000\076\000\000\000\035\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\074\000\000\000\
    \000\000\000\000\000\000\041\000\000\000\000\000\000\000\000\000\
    \058\000\000\000\000\000\071\000\000\000\107\000\000\000\255\255\
    \000\000\000\000\076\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\048\000\074\000\048\000\
    \000\000\000\000\047\000\047\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\000\000\000\000\
    \001\000\255\255\000\000\000\000\255\255\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\000\000\
    \000\000\000\000\000\000\030\000\000\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\000\000\000\000\000\000\000\000\043\000\000\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\000\000\255\255\000\000\000\000\000\000\000\000\
    \255\255\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\105\000\000\000\255\255\000\000\043\000\
    \000\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\047\000\047\000\047\000\045\000\
    \046\000\045\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\045\000\000\000\045\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\047\000\047\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\045\000\
    \046\000\045\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \085\000\045\000\085\000\045\000\000\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\072\000\
    \072\000\072\000\072\000\072\000\072\000\072\000\072\000\072\000\
    \072\000\082\000\082\000\082\000\082\000\082\000\082\000\082\000\
    \082\000\082\000\082\000\080\000\081\000\080\000\072\000\000\000\
    \073\000\073\000\073\000\073\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\042\000\071\000\112\000\101\000\
    \000\000\000\000\112\000\080\000\081\000\080\000\078\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\078\000\078\000\078\000\
    \000\000\000\000\000\000\000\000\103\000\112\000\000\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\071\000\000\000\072\000\
    \000\000\075\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \075\000\073\000\073\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\038\000\071\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\077\000\078\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\078\000\078\000\078\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\102\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\000\000\071\000\000\000\
    \000\000\083\000\079\000\083\000\000\000\077\000\082\000\082\000\
    \082\000\082\000\082\000\082\000\082\000\082\000\082\000\082\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\000\000\000\000\110\000\
    \255\255\000\000\079\000\082\000\082\000\082\000\082\000\082\000\
    \082\000\082\000\082\000\082\000\082\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\080\000\
    \090\000\080\000\000\000\000\000\000\000\091\000\000\000\111\000\
    \255\255\086\000\000\000\086\000\105\000\255\255\000\000\000\000\
    \000\000\093\000\093\000\093\000\093\000\093\000\093\000\093\000\
    \093\000\093\000\093\000\000\000\000\000\105\000\255\255\080\000\
    \000\000\080\000\093\000\093\000\093\000\093\000\093\000\093\000\
    \000\000\086\000\112\000\086\000\000\000\000\000\112\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\090\000\000\000\000\000\000\000\000\000\000\000\
    \090\000\112\000\000\000\000\000\090\000\000\000\000\000\255\255\
    \255\255\000\000\000\000\000\000\090\000\000\000\000\000\000\000\
    \090\000\000\000\090\000\089\000\094\000\094\000\094\000\094\000\
    \094\000\094\000\094\000\094\000\094\000\094\000\000\000\255\255\
    \255\255\038\000\039\000\000\000\000\000\094\000\094\000\094\000\
    \094\000\094\000\094\000\095\000\095\000\095\000\095\000\095\000\
    \095\000\095\000\095\000\095\000\095\000\033\000\000\000\000\000\
    \000\000\000\000\033\000\000\000\095\000\095\000\095\000\095\000\
    \095\000\095\000\096\000\096\000\096\000\096\000\096\000\096\000\
    \096\000\096\000\096\000\096\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\096\000\096\000\096\000\096\000\096\000\
    \096\000\255\255\255\255\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\033\000\
    \000\000\000\000\000\000\000\000\000\000\033\000\000\000\000\000\
    \000\000\033\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\033\000\000\000\000\000\000\000\033\000\000\000\033\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\039\000\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\255\255\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\040\000\034\000\000\000\255\255\255\255\
    \034\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\034\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \017\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \010\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\011\000\
    \019\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\013\000\013\000\014\000\014\000\020\000\
    \020\000\021\000\022\000\023\000\024\000\036\000\011\000\037\000\
    \025\000\026\000\041\000\064\000\012\000\011\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \025\000\022\000\069\000\026\000\032\000\067\000\067\000\087\000\
    \090\000\032\000\091\000\012\000\096\000\104\000\011\000\106\000\
    \107\000\111\000\012\000\255\255\255\255\011\000\255\255\032\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\012\000\255\255\
    \255\255\255\255\255\255\036\000\255\255\255\255\255\255\255\255\
    \021\000\255\255\255\255\012\000\255\255\104\000\255\255\106\000\
    \255\255\255\255\012\000\027\000\027\000\027\000\027\000\027\000\
    \027\000\027\000\027\000\027\000\027\000\046\000\012\000\046\000\
    \255\255\255\255\046\000\046\000\046\000\046\000\046\000\046\000\
    \046\000\046\000\046\000\046\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\255\255\255\255\
    \000\000\040\000\255\255\255\255\010\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\255\255\
    \255\255\255\255\255\255\030\000\255\255\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\255\255\255\255\255\255\255\255\031\000\255\255\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\255\255\036\000\255\255\255\255\255\255\255\255\
    \041\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\104\000\255\255\106\000\255\255\043\000\
    \255\255\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\047\000\047\000\047\000\044\000\
    \044\000\044\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\047\000\255\255\047\000\048\000\048\000\048\000\048\000\
    \048\000\048\000\048\000\048\000\048\000\048\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\044\000\
    \044\000\044\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \071\000\047\000\071\000\047\000\255\255\071\000\071\000\071\000\
    \071\000\071\000\071\000\071\000\071\000\071\000\071\000\072\000\
    \072\000\072\000\072\000\072\000\072\000\072\000\072\000\072\000\
    \072\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\072\000\072\000\072\000\073\000\255\255\
    \073\000\073\000\073\000\073\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\085\000\085\000\085\000\085\000\085\000\085\000\
    \085\000\085\000\085\000\085\000\099\000\073\000\110\000\099\000\
    \255\255\255\255\110\000\072\000\072\000\072\000\074\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \255\255\255\255\255\255\255\255\099\000\110\000\255\255\074\000\
    \074\000\074\000\074\000\074\000\074\000\073\000\255\255\075\000\
    \255\255\075\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \075\000\075\000\075\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\110\000\075\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\075\000\078\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\078\000\078\000\078\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\099\000\078\000\
    \078\000\078\000\078\000\078\000\078\000\255\255\075\000\255\255\
    \255\255\081\000\078\000\081\000\255\255\075\000\081\000\081\000\
    \081\000\081\000\081\000\081\000\081\000\081\000\081\000\081\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\078\000\
    \078\000\078\000\078\000\078\000\078\000\255\255\255\255\108\000\
    \109\000\255\255\078\000\082\000\082\000\082\000\082\000\082\000\
    \082\000\082\000\082\000\082\000\082\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\082\000\
    \088\000\082\000\255\255\255\255\255\255\088\000\255\255\108\000\
    \109\000\084\000\255\255\084\000\108\000\109\000\255\255\255\255\
    \255\255\089\000\089\000\089\000\089\000\089\000\089\000\089\000\
    \089\000\089\000\089\000\255\255\255\255\108\000\109\000\082\000\
    \255\255\082\000\089\000\089\000\089\000\089\000\089\000\089\000\
    \255\255\084\000\112\000\084\000\255\255\255\255\112\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\088\000\255\255\255\255\255\255\255\255\255\255\
    \088\000\112\000\255\255\255\255\088\000\255\255\255\255\100\000\
    \101\000\255\255\255\255\255\255\088\000\255\255\255\255\255\255\
    \088\000\255\255\088\000\088\000\093\000\093\000\093\000\093\000\
    \093\000\093\000\093\000\093\000\093\000\093\000\255\255\100\000\
    \101\000\112\000\099\000\255\255\255\255\093\000\093\000\093\000\
    \093\000\093\000\093\000\094\000\094\000\094\000\094\000\094\000\
    \094\000\094\000\094\000\094\000\094\000\102\000\255\255\255\255\
    \255\255\255\255\102\000\255\255\094\000\094\000\094\000\094\000\
    \094\000\094\000\095\000\095\000\095\000\095\000\095\000\095\000\
    \095\000\095\000\095\000\095\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\095\000\095\000\095\000\095\000\095\000\
    \095\000\100\000\101\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\102\000\
    \255\255\255\255\255\255\255\255\255\255\102\000\255\255\255\255\
    \255\255\102\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\102\000\255\255\255\255\255\255\102\000\255\255\102\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\108\000\109\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\100\000\101\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\102\000";
  Lexing.lex_base_code = 
   "";
  Lexing.lex_backtrk_code = 
   "";
  Lexing.lex_default_code = 
   "";
  Lexing.lex_trans_code = 
   "";
  Lexing.lex_check_code = 
   "";
  Lexing.lex_code = 
   "";
}

let rec token lexbuf =
    __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 188 "java/java_lexer.mll"
      ( token lexbuf )
# 513 "java/java_lexer.ml"

  | 1 ->
# 190 "java/java_lexer.mll"
      ( newline lexbuf; token lexbuf )
# 518 "java/java_lexer.ml"

  | 2 ->
# 192 "java/java_lexer.mll"
      ( let loc = lexeme_start_p lexbuf in
	Buffer.clear buf; ANNOT(loc,annot lexbuf) )
# 524 "java/java_lexer.ml"

  | 3 ->

  let a = Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 3) (lexbuf.Lexing.lex_curr_pos + -1) in
# 195 "java/java_lexer.mll"
      ( let loc = lexeme_start_p lexbuf in
	newline lexbuf;	ANNOT(loc,a) )
# 532 "java/java_lexer.ml"

  | 4 ->
# 198 "java/java_lexer.mll"
      ( comment lexbuf; token lexbuf )
# 537 "java/java_lexer.ml"

  | 5 ->
# 200 "java/java_lexer.mll"
      ( newline lexbuf; token lexbuf )
# 542 "java/java_lexer.ml"

  | 6 ->

  let id = Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) lexbuf.Lexing.lex_curr_pos in
# 202 "java/java_lexer.mll"
      ( special_id lexbuf id )
# 549 "java/java_lexer.ml"

  | 7 ->

  let id = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 204 "java/java_lexer.mll"
      ( id_or_kw id )
# 556 "java/java_lexer.ml"

  | 8 ->
# 206 "java/java_lexer.mll"
      ( SEMICOLON )
# 561 "java/java_lexer.ml"

  | 9 ->
# 208 "java/java_lexer.mll"
      ( COMMA )
# 566 "java/java_lexer.ml"

  | 10 ->
# 210 "java/java_lexer.mll"
      ( DOT )
# 571 "java/java_lexer.ml"

  | 11 ->
# 212 "java/java_lexer.mll"
      ( PLUS )
# 576 "java/java_lexer.ml"

  | 12 ->
# 214 "java/java_lexer.mll"
      ( MINUS )
# 581 "java/java_lexer.ml"

  | 13 ->
# 216 "java/java_lexer.mll"
      ( PLUSPLUS )
# 586 "java/java_lexer.ml"

  | 14 ->
# 218 "java/java_lexer.mll"
      ( MINUSMINUS )
# 591 "java/java_lexer.ml"

  | 15 ->
# 220 "java/java_lexer.mll"
      ( STAR )
# 596 "java/java_lexer.ml"

  | 16 ->
# 222 "java/java_lexer.mll"
      ( SLASH )
# 601 "java/java_lexer.ml"

  | 17 ->
# 224 "java/java_lexer.mll"
      ( PERCENT )
# 606 "java/java_lexer.ml"

  | 18 ->
# 226 "java/java_lexer.mll"
      ( AMPERSAND )
# 611 "java/java_lexer.ml"

  | 19 ->
# 228 "java/java_lexer.mll"
      ( VERTICALBAR )
# 616 "java/java_lexer.ml"

  | 20 ->
# 230 "java/java_lexer.mll"
      ( AMPERSANDAMPERSAND )
# 621 "java/java_lexer.ml"

  | 21 ->
# 232 "java/java_lexer.mll"
      ( VERTICALBARVERTICALBAR )
# 626 "java/java_lexer.ml"

  | 22 ->
# 234 "java/java_lexer.mll"
      ( EQGT )
# 631 "java/java_lexer.ml"

  | 23 ->
# 236 "java/java_lexer.mll"
      ( BANG )
# 636 "java/java_lexer.ml"

  | 24 ->
# 238 "java/java_lexer.mll"
      ( TILDA )
# 641 "java/java_lexer.ml"

  | 25 ->
# 240 "java/java_lexer.mll"
      ( CARET )
# 646 "java/java_lexer.ml"

  | 26 ->
# 242 "java/java_lexer.mll"
      ( QUESTIONMARK )
# 651 "java/java_lexer.ml"

  | 27 ->
# 244 "java/java_lexer.mll"
      ( COLON )
# 656 "java/java_lexer.ml"

  | 28 ->
# 246 "java/java_lexer.mll"
      ( SHIFT Blsl )
# 661 "java/java_lexer.ml"

  | 29 ->
# 248 "java/java_lexer.mll"
      ( SHIFT Blsr )
# 666 "java/java_lexer.ml"

  | 30 ->
# 250 "java/java_lexer.mll"
      ( SHIFT Basr )
# 671 "java/java_lexer.ml"

  | 31 ->
# 252 "java/java_lexer.mll"
      ( EQ )
# 676 "java/java_lexer.ml"

  | 32 ->
# 254 "java/java_lexer.mll"
      ( ASSIGNOP Bmul )
# 681 "java/java_lexer.ml"

  | 33 ->
# 256 "java/java_lexer.mll"
      ( ASSIGNOP Bdiv )
# 686 "java/java_lexer.ml"

  | 34 ->
# 258 "java/java_lexer.mll"
      ( ASSIGNOP Bmod )
# 691 "java/java_lexer.ml"

  | 35 ->
# 260 "java/java_lexer.mll"
      ( ASSIGNOP Badd )
# 696 "java/java_lexer.ml"

  | 36 ->
# 262 "java/java_lexer.mll"
      ( ASSIGNOP Bsub )
# 701 "java/java_lexer.ml"

  | 37 ->
# 264 "java/java_lexer.mll"
      ( ASSIGNOP Blsl )
# 706 "java/java_lexer.ml"

  | 38 ->
# 266 "java/java_lexer.mll"
      ( ASSIGNOP Blsr )
# 711 "java/java_lexer.ml"

  | 39 ->
# 268 "java/java_lexer.mll"
      ( ASSIGNOP Basr )
# 716 "java/java_lexer.ml"

  | 40 ->
# 270 "java/java_lexer.mll"
      ( ASSIGNOP Bbwand )
# 721 "java/java_lexer.ml"

  | 41 ->
# 272 "java/java_lexer.mll"
      ( ASSIGNOP Bbwxor )
# 726 "java/java_lexer.ml"

  | 42 ->
# 274 "java/java_lexer.mll"
      ( ASSIGNOP Bbwor )
# 731 "java/java_lexer.ml"

  | 43 ->
# 276 "java/java_lexer.mll"
      ( COMP Bgt )
# 736 "java/java_lexer.ml"

  | 44 ->
# 278 "java/java_lexer.mll"
      ( COMP Blt )
# 741 "java/java_lexer.ml"

  | 45 ->
# 280 "java/java_lexer.mll"
      ( COMP Ble )
# 746 "java/java_lexer.ml"

  | 46 ->
# 282 "java/java_lexer.mll"
      ( COMP Bge )
# 751 "java/java_lexer.ml"

  | 47 ->
# 284 "java/java_lexer.mll"
      ( EQOP Beq )
# 756 "java/java_lexer.ml"

  | 48 ->
# 286 "java/java_lexer.mll"
      ( EQOP Bne )
# 761 "java/java_lexer.ml"

  | 49 ->

  let n = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 291 "java/java_lexer.mll"
      ( INTEGER n )
# 768 "java/java_lexer.ml"

  | 50 ->

  let n = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 296 "java/java_lexer.mll"
      ( INTEGER ("0o" ^ n) )
# 775 "java/java_lexer.ml"

  | 51 ->

  let n = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 301 "java/java_lexer.mll"
    ( INTEGER n )
# 782 "java/java_lexer.ml"

  | 52 ->
# 306 "java/java_lexer.mll"
      ( REAL (lexeme lexbuf) )
# 787 "java/java_lexer.ml"

  | 53 ->
# 309 "java/java_lexer.mll"
      ( REAL (lexeme lexbuf) )
# 792 "java/java_lexer.ml"

  | 54 ->
# 312 "java/java_lexer.mll"
      ( REAL (lexeme lexbuf) )
# 797 "java/java_lexer.ml"

  | 55 ->

  let s = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_start_pos + 3) in
# 317 "java/java_lexer.mll"
      ( CHARACTER s )
# 804 "java/java_lexer.ml"

  | 56 ->

  let s = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_start_pos + 4) in
# 320 "java/java_lexer.mll"
      ( CHARACTER s )
# 811 "java/java_lexer.ml"

  | 57 ->

  let s = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_start_pos + 8) in
# 324 "java/java_lexer.mll"
      ( CHARACTER s )
# 818 "java/java_lexer.ml"

  | 58 ->
# 327 "java/java_lexer.mll"
      ( LEFTPAR )
# 823 "java/java_lexer.ml"

  | 59 ->
# 329 "java/java_lexer.mll"
      ( RIGHTPAR )
# 828 "java/java_lexer.ml"

  | 60 ->
# 331 "java/java_lexer.mll"
      ( LEFTBRACE )
# 833 "java/java_lexer.ml"

  | 61 ->
# 333 "java/java_lexer.mll"
      ( RIGHTBRACE )
# 838 "java/java_lexer.ml"

  | 62 ->
# 335 "java/java_lexer.mll"
      ( LEFTBRACKET )
# 843 "java/java_lexer.ml"

  | 63 ->
# 337 "java/java_lexer.mll"
      ( RIGHTBRACKET )
# 848 "java/java_lexer.ml"

  | 64 ->
# 339 "java/java_lexer.mll"
      ( Buffer.clear buf; STRING(string lexbuf) )
# 853 "java/java_lexer.ml"

  | 65 ->
# 341 "java/java_lexer.mll"
      ( lex_error lexbuf ("unexpected char `" ^ lexeme lexbuf ^ "'") )
# 858 "java/java_lexer.ml"

  | 66 ->
# 343 "java/java_lexer.mll"
      ( EOF )
# 863 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf __ocaml_lex_state

and string lexbuf =
    __ocaml_lex_string_rec lexbuf 99
and __ocaml_lex_string_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 347 "java/java_lexer.mll"
      ( Buffer.contents buf )
# 874 "java/java_lexer.ml"

  | 1 ->
# 349 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf);
	string lexbuf )
# 880 "java/java_lexer.ml"

  | 2 ->
# 352 "java/java_lexer.mll"
      ( lex_error lexbuf "unknown escape sequence in string" )
# 885 "java/java_lexer.ml"

  | 3 ->
# 354 "java/java_lexer.mll"
      ( (* no \n anymore in strings since java 1.4 *)
	lex_error lexbuf "string not terminated"; )
# 891 "java/java_lexer.ml"

  | 4 ->
# 357 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf); string lexbuf )
# 896 "java/java_lexer.ml"

  | 5 ->
# 359 "java/java_lexer.mll"
      ( lex_error lexbuf "string not terminated" )
# 901 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_string_rec lexbuf __ocaml_lex_state

and comment lexbuf =
    __ocaml_lex_comment_rec lexbuf 104
and __ocaml_lex_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 363 "java/java_lexer.mll"
      ( () )
# 912 "java/java_lexer.ml"

  | 1 ->
# 365 "java/java_lexer.mll"
      ( newline lexbuf; comment lexbuf )
# 917 "java/java_lexer.ml"

  | 2 ->
# 367 "java/java_lexer.mll"
      ( comment lexbuf )
# 922 "java/java_lexer.ml"

  | 3 ->
# 369 "java/java_lexer.mll"
      ( comment lexbuf )
# 927 "java/java_lexer.ml"

  | 4 ->
# 371 "java/java_lexer.mll"
      ( lex_error lexbuf "comment not terminated" )
# 932 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_rec lexbuf __ocaml_lex_state

and annot lexbuf =
    __ocaml_lex_annot_rec lexbuf 108
and __ocaml_lex_annot_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 375 "java/java_lexer.mll"
      ( Buffer.contents buf )
# 943 "java/java_lexer.ml"

  | 1 ->
# 377 "java/java_lexer.mll"
      ( newline lexbuf;  
	Buffer.add_string buf (lexeme lexbuf);
	annot lexbuf )
# 950 "java/java_lexer.ml"

  | 2 ->

  let s = Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_curr_pos + -1) in
# 381 "java/java_lexer.mll"
      ( newline lexbuf;  
	Buffer.add_string buf s;
	Buffer.add_char buf ' ';
	annot lexbuf )
# 960 "java/java_lexer.ml"

  | 3 ->
# 386 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf);
	annot lexbuf )
# 966 "java/java_lexer.ml"

  | 4 ->
# 389 "java/java_lexer.mll"
      ( Buffer.add_string buf (lexeme lexbuf);
	annot lexbuf )
# 972 "java/java_lexer.ml"

  | 5 ->
# 392 "java/java_lexer.mll"
      ( lex_error lexbuf "annotation not terminated" )
# 977 "java/java_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_annot_rec lexbuf __ocaml_lex_state

;;

# 394 "java/java_lexer.mll"
 

let dotdot_mem = ref false
 
let next_token lexbuf =
  if !dotdot_mem then
    begin
      dotdot_mem := false;
      DOTDOT
    end
  else
    begin
      try
	token lexbuf
      with
	Dotdot(n) ->
	  dotdot_mem := true;
	  INTEGER n
    end

  let parse f c =
    let lb = from_channel c in
    lb.lex_curr_p <- { lb.lex_curr_p with pos_fname = f };
    try
      Java_parser.compilation_unit next_token lb
    with Parsing.Parse_error ->
      Java_options.parsing_error (loc lb) ""


# 1013 "java/java_lexer.ml"
